import ExpoModulesCore
import BanubaVideoEditorSDK
import VideoEditor
import VEExportSDK
import BanubaUtilities
import Foundation
import UIKit
import Photos
import BSImagePicker
import BanubaLicenseServicingSDK

typealias TimerOptionConfiguration = TimerConfiguration.TimerOptionConfiguration

public class ExpoBanubaModule: Module {
    var delegate = ExpoBanubaDelegate()
    public func definition() -> ModuleDefinition {
        Name("ExpoBanuba")
        
        AsyncFunction("initVideoEditor") { (token: String, promise: Promise) in
            delegate.openVideoEditorDefault(self, token)
        }.runOnQueue(.main)
        
        AsyncFunction("openVideoEditor") { (promise: Promise) in
            // delegate.openVideoEditorDefault(self)
        }.runOnQueue(.main)
        
        View(ExpoBanubaDelegate.self){
        }
    }
}

class ExpoBanubaDelegate: ExpoView, BanubaVideoEditorDelegate {
  // MARK: - VideoEditorSDK
  private var videoEditorModule: VideoEditorModule?
  
  // Use “true” if you want users could restore the last video editing session.
  private let restoreLastVideoEditingSession: Bool = false

  // public required init(appContext: AppContext? = nil) {
  //   super.init(appContext: appContext)
  //   addSubview(self)
  // }

  // MARK: - Handle BanubaVideoEditor callbacks
  func videoEditorDidCancel(_ videoEditor: BanubaVideoEditor) {
    if restoreLastVideoEditingSession == false {
      videoEditor.clearSessionData()
    }
    videoEditor.dismissVideoEditor(animated: true, completion: nil)
  }
  
  func videoEditorDone(_ videoEditor: BanubaVideoEditor) {
    videoEditor.dismissVideoEditor(animated: true) { [weak self] in
      self?.exportVideo(videoEditor: videoEditor)
    }
  }

  func openVideoEditorDefault(_ sender: Any, _ token: String) {
      let musicTrackPreset: MediaTrack? = nil
      // Uncomment to apply custom audio track in video editor
      //let musicTrackPreset = prepareMusicTrack(audioFileName: "short_music_20.wav")
      
      guard let viewController = RCTPresentedViewController() else {
          print("Error when trying to get the viewController");
          return;
      }
      
      let launchConfig = VideoEditorLaunchConfig(
        entryPoint: .camera,
        hostController: viewController,
        musicTrack: musicTrackPreset, // Paste a music track as a track preset at the camera screen to record video with music
        animated: true
      )
      checkLicenseAndOpenVideoEditor(with: launchConfig, token)
  }
  
  private func checkLicenseAndOpenVideoEditor(with launchConfig: VideoEditorLaunchConfig, _ token: String) {
    // Deallocate any active instances of both editors to free used resources
    // and to prevent "You are trying to create the second instance of the singleton." crash
    videoEditorModule = nil
    
    videoEditorModule = VideoEditorModule(token: token)
    
    guard let videoEditorSDK = videoEditorModule?.videoEditorSDK else {
      return
    }

    videoEditorSDK.delegate = self
    videoEditorSDK.getLicenseState(completion: { [weak self] isValid in
      if isValid {
        print("✅ License is active, all good")
        self?.videoEditorModule?.presentVideoEditor(with: launchConfig)
      } else {
        print("❌ License is either revoked or expired")
      }
    })
  }

}

// MARK: - Export example
extension ExpoBanubaDelegate {
  func exportVideo(videoEditor: BanubaVideoEditor) {
    guard let videoEditorModule else { return }
    let progressViewController = videoEditorModule.createProgressViewController()
    progressViewController.cancelHandler = { videoEditor.stopExport() }
    guard let viewController = RCTPresentedViewController() else {
      print("Error when trying to get the viewController");
      return;
    }
    viewController.present(progressViewController, animated: true)
    
    let manager = FileManager.default
    let exportedVideoFileName = "tmp.mov"
    let videoURL = manager.temporaryDirectory.appendingPathComponent(exportedVideoFileName)
    if manager.fileExists(atPath: videoURL.path) {
      try? manager.removeItem(at: videoURL)
    }
    
    let exportConfiguration = videoEditorModule.createExportConfiguration(destFile: videoURL)
    
    videoEditor.export(
      using: exportConfiguration,
      exportProgress: { [weak progressViewController] progress in
        DispatchQueue.main.async {
          progressViewController?.updateProgressView(with: Float(progress))
        }
      },
      completion: { [weak self, weak progressViewController] error, exportCoverImages in
        DispatchQueue.main.async {
          // Hide progress view
          progressViewController?.dismiss(animated: true) {
            // Clear video editor session data
            if self?.restoreLastVideoEditingSession == false {
              videoEditor.clearSessionData()
            }
            if error == nil {
              self?.showExportResult(
                videoUrl: videoURL,
                exportCoverImages: exportCoverImages
              )
            }
          }
        }
    })
  }
  
  func showExportResult(
    videoUrl: URL,
    exportCoverImages: ExportCoverImages?
  ) {
    // The popup is used to demo export result in a various ways.
    // It is not required to copy and paste this approach to your project.
    
    let alertController = UIAlertController(
      title: nil,
      message: nil,
      preferredStyle: .alert
    )
    
    let previewButton = UIAlertAction(title: "Play Video", style: .default) { _ in
      /// If you want to play exported video
      self.playVideoAtURL(videoUrl)
    }
    let shareButton = UIAlertAction(title: "Open Sharing", style: .default) { [weak self] _ in
      /// If you want to share exported video
      self?.showSharingScreen(
        videoUrl: videoUrl,
        exportCoverImages: exportCoverImages
      )
    }
    let cancelButton = UIAlertAction(title: "Close", style: .destructive)
    
    alertController.addAction(previewButton)
    alertController.addAction(shareButton)
    alertController.addAction(cancelButton)
      guard let viewController = RCTPresentedViewController() else {
        print("Error when trying to get the viewController");
        return;
      }
      viewController.present(alertController, animated: true)
  }
  
  private func showSharingScreen(videoUrl: URL, exportCoverImages: ExportCoverImages?) {
    // Set up sharing configurations
    //SharingScreenConfiguration.sharingModels - describes what kind of sharing services are available at sharing screen.
    //SharingScreenConfiguration.facebookId - is a required option for Facebook reels and stories.
    guard let config = videoEditorModule?.videoEditorSDK?.currentConfiguration.sharingScreenConfiguration else { return }
    
    guard let viewController = RCTPresentedViewController() else {
      print("Error when trying to get the viewController");
      return;
    }
    BanubaVideoEditor.presentSharingViewController(
      from: viewController,
      configuration: config,
      mainVideoUrl: videoUrl,
      videoUrls: [videoUrl],
      previewImage: exportCoverImages?.coverImage ?? UIImage(),
      animated: true,
      completion: nil
    )
  }
  
  /// For demonstration purpose let's play exported video
  private func playVideoAtURL(_ videoURL: URL) {
    let player = AVPlayer(url: videoURL)
    let playerController = AVPlayerViewController()
    playerController.player = player
      guard let viewController = RCTPresentedViewController() else {
        print("Error when trying to get the viewController");
        return;
      }
      viewController.present(playerController, animated: true) {
      player.play()
    }
  }
}
